package dev.shadowsoffire.apotheosis.ench;

import dev.shadowsoffire.apotheosis.util.ApothMiscUtil;
import dev.shadowsoffire.placebo.color.GradientColor;
import com.mojang.datafixers.types.Type;
import java.util.Set;
import com.google.common.collect.ImmutableSet;
import net.minecraft.world.level.block.entity.BlockEntityType;
import net.minecraft.world.item.ItemStack;
import net.minecraft.resources.ResourceLocation;
import net.minecraft.network.chat.Component;
import net.minecraft.world.item.CreativeModeTab;
import dev.shadowsoffire.apotheosis.ench.enchantments.TemptingEnchant;
import dev.shadowsoffire.apotheosis.ench.enchantments.StableFootingEnchant;
import dev.shadowsoffire.apotheosis.ench.anvil.SplittingEnchant;
import dev.shadowsoffire.apotheosis.ench.enchantments.SpearfishingEnchant;
import dev.shadowsoffire.apotheosis.ench.enchantments.ShieldBashEnchant;
import dev.shadowsoffire.apotheosis.ench.enchantments.masterwork.ScavengerEnchant;
import dev.shadowsoffire.apotheosis.ench.enchantments.ReflectiveEnchant;
import dev.shadowsoffire.apotheosis.ench.enchantments.ReboundingEnchant;
import dev.shadowsoffire.apotheosis.ench.anvil.ObliterationEnchant;
import dev.shadowsoffire.apotheosis.ench.enchantments.NaturesBlessingEnchant;
import dev.shadowsoffire.apotheosis.ench.enchantments.twisted.MinersFervorEnchant;
import dev.shadowsoffire.apotheosis.ench.enchantments.corrupted.LifeMendingEnchant;
import dev.shadowsoffire.apotheosis.ench.enchantments.masterwork.KnowledgeEnchant;
import dev.shadowsoffire.apotheosis.ench.enchantments.InertEnchantment;
import dev.shadowsoffire.apotheosis.ench.enchantments.IcyThornsEnchant;
import dev.shadowsoffire.apotheosis.ench.enchantments.masterwork.GrowthSerumEnchant;
import dev.shadowsoffire.apotheosis.ench.enchantments.twisted.ExploitationEnchant;
import dev.shadowsoffire.apotheosis.ench.enchantments.masterwork.EndlessQuiverEnchant;
import dev.shadowsoffire.apotheosis.ench.enchantments.masterwork.EarthsBoonEnchant;
import dev.shadowsoffire.apotheosis.ench.enchantments.masterwork.CrescendoEnchant;
import dev.shadowsoffire.apotheosis.ench.enchantments.ChromaticEnchant;
import dev.shadowsoffire.apotheosis.ench.enchantments.masterwork.ChainsawEnchant;
import dev.shadowsoffire.apotheosis.ench.enchantments.corrupted.BerserkersFuryEnchant;
import net.minecraft.world.item.enchantment.EnchantmentCategory;
import net.minecraft.world.item.Items;
import net.minecraft.world.item.Rarity;
import dev.shadowsoffire.apotheosis.ench.objects.ScrappingTomeItem;
import net.minecraft.world.item.Item;
import dev.shadowsoffire.apotheosis.ench.objects.ImprovedScrappingTomeItem;
import dev.shadowsoffire.apotheosis.ench.objects.ExtractionTomeItem;
import dev.shadowsoffire.apotheosis.ench.objects.GlowyBlockItem;
import dev.shadowsoffire.apotheosis.ench.objects.TomeItem;
import net.minecraft.world.item.BlockItem;
import dev.shadowsoffire.apotheosis.ench.library.EnchLibraryTile;
import net.minecraft.core.particles.ParticleTypes;
import dev.shadowsoffire.apotheosis.ench.objects.FilteringShelfBlock;
import dev.shadowsoffire.apotheosis.ench.objects.TreasureShelfBlock;
import net.minecraft.core.particles.ParticleOptions;
import java.util.function.Supplier;
import dev.shadowsoffire.apotheosis.ench.objects.TypedShelfBlock;
import dev.shadowsoffire.apotheosis.Apoth;
import net.minecraft.world.level.block.SoundType;
import net.minecraft.world.level.material.MapColor;
import net.minecraft.world.level.block.state.BlockBehaviour;
import dev.shadowsoffire.apotheosis.ench.library.EnchLibraryBlock;
import net.minecraft.world.level.block.Block;
import net.minecraftforge.registries.RegistryObject;
import dev.shadowsoffire.apotheosis.Apotheosis;
import dev.shadowsoffire.placebo.registry.DeferredHelper;

public class Ench
{
    private static final DeferredHelper R;
    
    public static void bootstrap() {
        Blocks.bootstrap();
        Items.bootstrap();
        Enchantments.bootstrap();
        Tabs.bootstrap();
        Tiles.bootstrap();
    }
    
    static {
        R = (DeferredHelper)Apotheosis.ModularDeferredHelper.create(() -> Apotheosis.enableEnch);
    }
    
    public static final class Blocks
    {
        public static final RegistryObject<Block> BEESHELF;
        public static final RegistryObject<Block> BLAZING_HELLSHELF;
        public static final RegistryObject<Block> CRYSTAL_SEASHELF;
        public static final RegistryObject<Block> DEEPSHELF;
        public static final RegistryObject<Block> DORMANT_DEEPSHELF;
        public static final RegistryObject<Block> DRACONIC_ENDSHELF;
        public static final RegistryObject<Block> ECHOING_DEEPSHELF;
        public static final RegistryObject<Block> ECHOING_SCULKSHELF;
        public static final RegistryObject<EnchLibraryBlock> ENDER_LIBRARY;
        public static final RegistryObject<Block> ENDSHELF;
        public static final RegistryObject<Block> GLOWING_HELLSHELF;
        public static final RegistryObject<Block> HEART_SEASHELF;
        public static final RegistryObject<Block> HELLSHELF;
        public static final RegistryObject<Block> INFUSED_HELLSHELF;
        public static final RegistryObject<Block> INFUSED_SEASHELF;
        public static final RegistryObject<EnchLibraryBlock> LIBRARY;
        public static final RegistryObject<Block> MELONSHELF;
        public static final RegistryObject<Block> PEARL_ENDSHELF;
        public static final RegistryObject<Block> RECTIFIER;
        public static final RegistryObject<Block> RECTIFIER_T2;
        public static final RegistryObject<Block> RECTIFIER_T3;
        public static final RegistryObject<Block> SEASHELF;
        public static final RegistryObject<Block> SIGHTSHELF;
        public static final RegistryObject<Block> SIGHTSHELF_T2;
        public static final RegistryObject<Block> SOUL_TOUCHED_DEEPSHELF;
        public static final RegistryObject<Block> SOUL_TOUCHED_SCULKSHELF;
        public static final RegistryObject<Block> STONESHELF;
        public static final RegistryObject<Block> FILTERING_SHELF;
        public static final RegistryObject<Block> TREASURE_SHELF;
        
        private static void bootstrap() {
        }
        
        private static RegistryObject<Block> sculkShelf(final String id) {
            return (RegistryObject<Block>)Ench.R.block(id, () -> new TypedShelfBlock.SculkShelfBlock(BlockBehaviour.Properties.m_284310_().m_284180_(MapColor.f_283927_).m_60918_(SoundType.f_56742_).m_60977_().m_60999_().m_60978_(3.5f), (Supplier)Apoth.Particles.ENCHANT_SCULK));
        }
        
        private static RegistryObject<Block> stoneShelf(final String id, final MapColor color, final float strength, final Supplier<? extends ParticleOptions> particle) {
            return (RegistryObject<Block>)Ench.R.block(id, () -> new TypedShelfBlock(BlockBehaviour.Properties.m_284310_().m_60999_().m_60918_(SoundType.f_56742_).m_284180_(color).m_60978_(strength), particle));
        }
        
        private static RegistryObject<Block> woodShelf(final String id, final MapColor color, final float strength, final Supplier<? extends ParticleOptions> particle) {
            return (RegistryObject<Block>)Ench.R.block(id, () -> new TypedShelfBlock(BlockBehaviour.Properties.m_284310_().m_60918_(SoundType.f_56736_).m_284180_(color).m_60978_(strength), particle));
        }
        
        static {
            BEESHELF = woodShelf("beeshelf", MapColor.f_283832_, 0.75f, () -> ParticleTypes.f_123809_);
            BLAZING_HELLSHELF = stoneShelf("blazing_hellshelf", MapColor.f_283927_, 1.5f, (Supplier<? extends ParticleOptions>)Apoth.Particles.ENCHANT_FIRE);
            CRYSTAL_SEASHELF = stoneShelf("crystal_seashelf", MapColor.f_283772_, 1.5f, (Supplier<? extends ParticleOptions>)Apoth.Particles.ENCHANT_WATER);
            DEEPSHELF = stoneShelf("deepshelf", MapColor.f_283927_, 2.5f, (Supplier<? extends ParticleOptions>)Apoth.Particles.ENCHANT_SCULK);
            DORMANT_DEEPSHELF = stoneShelf("dormant_deepshelf", MapColor.f_283927_, 2.5f, (Supplier<? extends ParticleOptions>)Apoth.Particles.ENCHANT_SCULK);
            DRACONIC_ENDSHELF = stoneShelf("draconic_endshelf", MapColor.f_283761_, 5.0f, (Supplier<? extends ParticleOptions>)Apoth.Particles.ENCHANT_END);
            ECHOING_DEEPSHELF = stoneShelf("echoing_deepshelf", MapColor.f_283927_, 2.5f, (Supplier<? extends ParticleOptions>)Apoth.Particles.ENCHANT_SCULK);
            ECHOING_SCULKSHELF = sculkShelf("echoing_sculkshelf");
            ENDER_LIBRARY = Ench.R.block("ender_library", () -> new EnchLibraryBlock(EnchLibraryTile.EnderLibraryTile::new, 31));
            ENDSHELF = stoneShelf("endshelf", MapColor.f_283761_, 4.5f, (Supplier<? extends ParticleOptions>)Apoth.Particles.ENCHANT_END);
            GLOWING_HELLSHELF = stoneShelf("glowing_hellshelf", MapColor.f_283927_, 1.5f, (Supplier<? extends ParticleOptions>)Apoth.Particles.ENCHANT_FIRE);
            HEART_SEASHELF = stoneShelf("heart_seashelf", MapColor.f_283772_, 1.5f, (Supplier<? extends ParticleOptions>)Apoth.Particles.ENCHANT_WATER);
            HELLSHELF = stoneShelf("hellshelf", MapColor.f_283927_, 1.5f, (Supplier<? extends ParticleOptions>)Apoth.Particles.ENCHANT_FIRE);
            INFUSED_HELLSHELF = stoneShelf("infused_hellshelf", MapColor.f_283927_, 1.5f, (Supplier<? extends ParticleOptions>)Apoth.Particles.ENCHANT_FIRE);
            INFUSED_SEASHELF = stoneShelf("infused_seashelf", MapColor.f_283772_, 1.5f, (Supplier<? extends ParticleOptions>)Apoth.Particles.ENCHANT_WATER);
            LIBRARY = Ench.R.block("library", () -> new EnchLibraryBlock(EnchLibraryTile.BasicLibraryTile::new, 16));
            MELONSHELF = woodShelf("melonshelf", MapColor.f_283784_, 0.75f, () -> ParticleTypes.f_123809_);
            PEARL_ENDSHELF = stoneShelf("pearl_endshelf", MapColor.f_283761_, 4.5f, (Supplier<? extends ParticleOptions>)Apoth.Particles.ENCHANT_END);
            RECTIFIER = stoneShelf("rectifier", MapColor.f_283772_, 1.5f, (Supplier<? extends ParticleOptions>)Apoth.Particles.ENCHANT_WATER);
            RECTIFIER_T2 = stoneShelf("rectifier_t2", MapColor.f_283927_, 1.5f, (Supplier<? extends ParticleOptions>)Apoth.Particles.ENCHANT_FIRE);
            RECTIFIER_T3 = stoneShelf("rectifier_t3", MapColor.f_283761_, 1.5f, (Supplier<? extends ParticleOptions>)Apoth.Particles.ENCHANT_END);
            SEASHELF = stoneShelf("seashelf", MapColor.f_283772_, 1.5f, (Supplier<? extends ParticleOptions>)Apoth.Particles.ENCHANT_WATER);
            SIGHTSHELF = stoneShelf("sightshelf", MapColor.f_283927_, 1.5f, (Supplier<? extends ParticleOptions>)Apoth.Particles.ENCHANT_FIRE);
            SIGHTSHELF_T2 = stoneShelf("sightshelf_t2", MapColor.f_283927_, 1.5f, (Supplier<? extends ParticleOptions>)Apoth.Particles.ENCHANT_FIRE);
            SOUL_TOUCHED_DEEPSHELF = stoneShelf("soul_touched_deepshelf", MapColor.f_283927_, 2.5f, (Supplier<? extends ParticleOptions>)Apoth.Particles.ENCHANT_SCULK);
            SOUL_TOUCHED_SCULKSHELF = sculkShelf("soul_touched_sculkshelf");
            STONESHELF = stoneShelf("stoneshelf", MapColor.f_283947_, 1.75f, () -> ParticleTypes.f_123809_);
            FILTERING_SHELF = Ench.R.block("filtering_shelf", () -> new FilteringShelfBlock(BlockBehaviour.Properties.m_284310_().m_284180_(MapColor.f_283772_).m_60918_(SoundType.f_56742_).m_60978_(1.75f).m_60999_()));
            TREASURE_SHELF = Ench.R.block("treasure_shelf", () -> new TreasureShelfBlock(BlockBehaviour.Properties.m_284310_().m_284180_(MapColor.f_283927_).m_60918_(SoundType.f_56742_).m_60978_(1.75f).m_60999_()));
        }
    }
    
    public static class Items
    {
        public static final RegistryObject<BlockItem> BEESHELF;
        public static final RegistryObject<BlockItem> BLAZING_HELLSHELF;
        public static final RegistryObject<TomeItem> BOOTS_TOME;
        public static final RegistryObject<TomeItem> BOW_TOME;
        public static final RegistryObject<TomeItem> CHESTPLATE_TOME;
        public static final RegistryObject<BlockItem> CRYSTAL_SEASHELF;
        public static final RegistryObject<GlowyBlockItem> DEEPSHELF;
        public static final RegistryObject<BlockItem> DORMANT_DEEPSHELF;
        public static final RegistryObject<BlockItem> DRACONIC_ENDSHELF;
        public static final RegistryObject<BlockItem> ECHOING_DEEPSHELF;
        public static final RegistryObject<BlockItem> ECHOING_SCULKSHELF;
        public static final RegistryObject<BlockItem> ENDER_LIBRARY;
        public static final RegistryObject<BlockItem> ENDSHELF;
        public static final RegistryObject<ExtractionTomeItem> EXTRACTION_TOME;
        public static final RegistryObject<TomeItem> FISHING_TOME;
        public static final RegistryObject<BlockItem> GLOWING_HELLSHELF;
        public static final RegistryObject<BlockItem> HEART_SEASHELF;
        public static final RegistryObject<BlockItem> HELLSHELF;
        public static final RegistryObject<TomeItem> HELMET_TOME;
        public static final RegistryObject<ImprovedScrappingTomeItem> IMPROVED_SCRAP_TOME;
        public static final RegistryObject<Item> INERT_TRIDENT;
        public static final RegistryObject<Item> INFUSED_BREATH;
        public static final RegistryObject<GlowyBlockItem> INFUSED_HELLSHELF;
        public static final RegistryObject<GlowyBlockItem> INFUSED_SEASHELF;
        public static final RegistryObject<TomeItem> LEGGINGS_TOME;
        public static final RegistryObject<BlockItem> LIBRARY;
        public static final RegistryObject<BlockItem> MELONSHELF;
        public static final RegistryObject<TomeItem> OTHER_TOME;
        public static final RegistryObject<BlockItem> PEARL_ENDSHELF;
        public static final RegistryObject<TomeItem> PICKAXE_TOME;
        public static final RegistryObject<Item> PRISMATIC_WEB;
        public static final RegistryObject<BlockItem> RECTIFIER;
        public static final RegistryObject<BlockItem> RECTIFIER_T2;
        public static final RegistryObject<BlockItem> RECTIFIER_T3;
        public static final RegistryObject<ScrappingTomeItem> SCRAP_TOME;
        public static final RegistryObject<BlockItem> SEASHELF;
        public static final RegistryObject<BlockItem> SIGHTSHELF;
        public static final RegistryObject<BlockItem> SIGHTSHELF_T2;
        public static final RegistryObject<BlockItem> SOUL_TOUCHED_DEEPSHELF;
        public static final RegistryObject<BlockItem> SOUL_TOUCHED_SCULKSHELF;
        public static final RegistryObject<BlockItem> STONESHELF;
        public static final RegistryObject<Item> WARDEN_TENDRIL;
        public static final RegistryObject<TomeItem> WEAPON_TOME;
        public static final RegistryObject<BlockItem> FILTERING_SHELF;
        public static final RegistryObject<BlockItem> TREASURE_SHELF;
        
        private static void bootstrap() {
        }
        
        static {
            BEESHELF = Ench.R.item("beeshelf", () -> {
                new BlockItem((Block)Blocks.BEESHELF.get(), new Item.Properties());
                return;
            });
            BLAZING_HELLSHELF = Ench.R.item("blazing_hellshelf", () -> {
                new BlockItem((Block)Blocks.BLAZING_HELLSHELF.get(), new Item.Properties());
                return;
            });
            BOOTS_TOME = Ench.R.item("boots_tome", () -> new TomeItem(net.minecraft.world.item.Items.f_42475_, EnchantmentCategory.ARMOR_FEET));
            BOW_TOME = Ench.R.item("bow_tome", () -> new TomeItem(net.minecraft.world.item.Items.f_42411_, EnchantmentCategory.BOW));
            CHESTPLATE_TOME = Ench.R.item("chestplate_tome", () -> new TomeItem(net.minecraft.world.item.Items.f_42473_, EnchantmentCategory.ARMOR_CHEST));
            CRYSTAL_SEASHELF = Ench.R.item("crystal_seashelf", () -> {
                new BlockItem((Block)Blocks.CRYSTAL_SEASHELF.get(), new Item.Properties());
                return;
            });
            DEEPSHELF = Ench.R.item("deepshelf", () -> {
                new GlowyBlockItem((Block)Blocks.DEEPSHELF.get(), new Item.Properties());
                return;
            });
            DORMANT_DEEPSHELF = Ench.R.item("dormant_deepshelf", () -> {
                new BlockItem((Block)Blocks.DORMANT_DEEPSHELF.get(), new Item.Properties());
                return;
            });
            DRACONIC_ENDSHELF = Ench.R.item("draconic_endshelf", () -> {
                new BlockItem((Block)Blocks.DRACONIC_ENDSHELF.get(), new Item.Properties());
                return;
            });
            ECHOING_DEEPSHELF = Ench.R.item("echoing_deepshelf", () -> {
                new BlockItem((Block)Blocks.ECHOING_DEEPSHELF.get(), new Item.Properties());
                return;
            });
            ECHOING_SCULKSHELF = Ench.R.item("echoing_sculkshelf", () -> {
                new BlockItem((Block)Blocks.ECHOING_SCULKSHELF.get(), new Item.Properties());
                return;
            });
            ENDER_LIBRARY = Ench.R.item("ender_library", () -> {
                new BlockItem((Block)Blocks.ENDER_LIBRARY.get(), new Item.Properties());
                return;
            });
            ENDSHELF = Ench.R.item("endshelf", () -> {
                new BlockItem((Block)Blocks.ENDSHELF.get(), new Item.Properties());
                return;
            });
            EXTRACTION_TOME = Ench.R.item("extraction_tome", (Supplier)ExtractionTomeItem::new);
            FISHING_TOME = Ench.R.item("fishing_tome", () -> new TomeItem(net.minecraft.world.item.Items.f_42523_, EnchantmentCategory.FISHING_ROD));
            GLOWING_HELLSHELF = Ench.R.item("glowing_hellshelf", () -> {
                new BlockItem((Block)Blocks.GLOWING_HELLSHELF.get(), new Item.Properties());
                return;
            });
            HEART_SEASHELF = Ench.R.item("heart_seashelf", () -> {
                new BlockItem((Block)Blocks.HEART_SEASHELF.get(), new Item.Properties());
                return;
            });
            HELLSHELF = Ench.R.item("hellshelf", () -> {
                new BlockItem((Block)Blocks.HELLSHELF.get(), new Item.Properties());
                return;
            });
            HELMET_TOME = Ench.R.item("helmet_tome", () -> new TomeItem(net.minecraft.world.item.Items.f_42472_, EnchantmentCategory.ARMOR_HEAD));
            IMPROVED_SCRAP_TOME = Ench.R.item("improved_scrap_tome", (Supplier)ImprovedScrappingTomeItem::new);
            INERT_TRIDENT = Ench.R.item("inert_trident", () -> {
                new Item(new Item.Properties().m_41487_(1));
                return;
            });
            INFUSED_BREATH = Ench.R.item("infused_breath", () -> {
                new Item(new Item.Properties().m_41497_(Rarity.EPIC));
                return;
            });
            INFUSED_HELLSHELF = Ench.R.item("infused_hellshelf", () -> {
                new GlowyBlockItem((Block)Blocks.INFUSED_HELLSHELF.get(), new Item.Properties());
                return;
            });
            INFUSED_SEASHELF = Ench.R.item("infused_seashelf", () -> {
                new GlowyBlockItem((Block)Blocks.INFUSED_SEASHELF.get(), new Item.Properties());
                return;
            });
            LEGGINGS_TOME = Ench.R.item("leggings_tome", () -> new TomeItem(net.minecraft.world.item.Items.f_42474_, EnchantmentCategory.ARMOR_LEGS));
            LIBRARY = Ench.R.item("library", () -> {
                new BlockItem((Block)Blocks.LIBRARY.get(), new Item.Properties());
                return;
            });
            MELONSHELF = Ench.R.item("melonshelf", () -> {
                new BlockItem((Block)Blocks.MELONSHELF.get(), new Item.Properties());
                return;
            });
            OTHER_TOME = Ench.R.item("other_tome", () -> new TomeItem(net.minecraft.world.item.Items.f_41852_, (EnchantmentCategory)null));
            PEARL_ENDSHELF = Ench.R.item("pearl_endshelf", () -> {
                new BlockItem((Block)Blocks.PEARL_ENDSHELF.get(), new Item.Properties());
                return;
            });
            PICKAXE_TOME = Ench.R.item("pickaxe_tome", () -> new TomeItem(net.minecraft.world.item.Items.f_42390_, EnchantmentCategory.DIGGER));
            PRISMATIC_WEB = Ench.R.item("prismatic_web", () -> {
                new Item(new Item.Properties());
                return;
            });
            RECTIFIER = Ench.R.item("rectifier", () -> {
                new BlockItem((Block)Blocks.RECTIFIER.get(), new Item.Properties());
                return;
            });
            RECTIFIER_T2 = Ench.R.item("rectifier_t2", () -> {
                new BlockItem((Block)Blocks.RECTIFIER_T2.get(), new Item.Properties());
                return;
            });
            RECTIFIER_T3 = Ench.R.item("rectifier_t3", () -> {
                new BlockItem((Block)Blocks.RECTIFIER_T3.get(), new Item.Properties());
                return;
            });
            SCRAP_TOME = Ench.R.item("scrap_tome", (Supplier)ScrappingTomeItem::new);
            SEASHELF = Ench.R.item("seashelf", () -> {
                new BlockItem((Block)Blocks.SEASHELF.get(), new Item.Properties());
                return;
            });
            SIGHTSHELF = Ench.R.item("sightshelf", () -> {
                new BlockItem((Block)Blocks.SIGHTSHELF.get(), new Item.Properties());
                return;
            });
            SIGHTSHELF_T2 = Ench.R.item("sightshelf_t2", () -> {
                new BlockItem((Block)Blocks.SIGHTSHELF_T2.get(), new Item.Properties());
                return;
            });
            SOUL_TOUCHED_DEEPSHELF = Ench.R.item("soul_touched_deepshelf", () -> {
                new BlockItem((Block)Blocks.SOUL_TOUCHED_DEEPSHELF.get(), new Item.Properties());
                return;
            });
            SOUL_TOUCHED_SCULKSHELF = Ench.R.item("soul_touched_sculkshelf", () -> {
                new BlockItem((Block)Blocks.SOUL_TOUCHED_SCULKSHELF.get(), new Item.Properties());
                return;
            });
            STONESHELF = Ench.R.item("stoneshelf", () -> {
                new BlockItem((Block)Blocks.STONESHELF.get(), new Item.Properties());
                return;
            });
            WARDEN_TENDRIL = Ench.R.item("warden_tendril", () -> {
                new Item(new Item.Properties());
                return;
            });
            WEAPON_TOME = Ench.R.item("weapon_tome", () -> new TomeItem(net.minecraft.world.item.Items.f_42388_, EnchantmentCategory.WEAPON));
            FILTERING_SHELF = Ench.R.item("filtering_shelf", () -> {
                new BlockItem((Block)Blocks.FILTERING_SHELF.get(), new Item.Properties().m_41497_(Rarity.UNCOMMON));
                return;
            });
            TREASURE_SHELF = Ench.R.item("treasure_shelf", () -> {
                new BlockItem((Block)Blocks.TREASURE_SHELF.get(), new Item.Properties().m_41497_(Rarity.UNCOMMON));
                return;
            });
        }
    }
    
    public static final class Enchantments
    {
        public static final RegistryObject<BerserkersFuryEnchant> BERSERKERS_FURY;
        public static final RegistryObject<ChainsawEnchant> CHAINSAW;
        public static final RegistryObject<ChromaticEnchant> CHROMATIC;
        public static final RegistryObject<CrescendoEnchant> CRESCENDO;
        public static final RegistryObject<EarthsBoonEnchant> EARTHS_BOON;
        public static final RegistryObject<EndlessQuiverEnchant> ENDLESS_QUIVER;
        public static final RegistryObject<ExploitationEnchant> EXPLOITATION;
        public static final RegistryObject<GrowthSerumEnchant> GROWTH_SERUM;
        public static final RegistryObject<IcyThornsEnchant> ICY_THORNS;
        public static final RegistryObject<InertEnchantment> INFUSION;
        public static final RegistryObject<KnowledgeEnchant> KNOWLEDGE;
        public static final RegistryObject<LifeMendingEnchant> LIFE_MENDING;
        public static final RegistryObject<MinersFervorEnchant> MINERS_FERVOR;
        public static final RegistryObject<NaturesBlessingEnchant> NATURES_BLESSING;
        public static final RegistryObject<ObliterationEnchant> OBLITERATION;
        public static final RegistryObject<ReboundingEnchant> REBOUNDING;
        public static final RegistryObject<ReflectiveEnchant> REFLECTIVE;
        public static final RegistryObject<ScavengerEnchant> SCAVENGER;
        public static final RegistryObject<ShieldBashEnchant> SHIELD_BASH;
        public static final RegistryObject<SpearfishingEnchant> SPEARFISHING;
        public static final RegistryObject<SplittingEnchant> SPLITTING;
        public static final RegistryObject<StableFootingEnchant> STABLE_FOOTING;
        public static final RegistryObject<TemptingEnchant> TEMPTING;
        
        private static void bootstrap() {
        }
        
        static {
            BERSERKERS_FURY = Ench.R.enchant("berserkers_fury", (Supplier)BerserkersFuryEnchant::new);
            CHAINSAW = Ench.R.enchant("chainsaw", (Supplier)ChainsawEnchant::new);
            CHROMATIC = Ench.R.enchant("chromatic", (Supplier)ChromaticEnchant::new);
            CRESCENDO = Ench.R.enchant("crescendo", (Supplier)CrescendoEnchant::new);
            EARTHS_BOON = Ench.R.enchant("earths_boon", (Supplier)EarthsBoonEnchant::new);
            ENDLESS_QUIVER = Ench.R.enchant("endless_quiver", (Supplier)EndlessQuiverEnchant::new);
            EXPLOITATION = Ench.R.enchant("exploitation", (Supplier)ExploitationEnchant::new);
            GROWTH_SERUM = Ench.R.enchant("growth_serum", (Supplier)GrowthSerumEnchant::new);
            ICY_THORNS = Ench.R.enchant("icy_thorns", (Supplier)IcyThornsEnchant::new);
            INFUSION = Ench.R.enchant("infusion", (Supplier)InertEnchantment::new);
            KNOWLEDGE = Ench.R.enchant("knowledge", (Supplier)KnowledgeEnchant::new);
            LIFE_MENDING = Ench.R.enchant("life_mending", (Supplier)LifeMendingEnchant::new);
            MINERS_FERVOR = Ench.R.enchant("miners_fervor", (Supplier)MinersFervorEnchant::new);
            NATURES_BLESSING = Ench.R.enchant("natures_blessing", (Supplier)NaturesBlessingEnchant::new);
            OBLITERATION = Ench.R.enchant("obliteration", (Supplier)ObliterationEnchant::new);
            REBOUNDING = Ench.R.enchant("rebounding", (Supplier)ReboundingEnchant::new);
            REFLECTIVE = Ench.R.enchant("reflective", (Supplier)ReflectiveEnchant::new);
            SCAVENGER = Ench.R.enchant("scavenger", (Supplier)ScavengerEnchant::new);
            SHIELD_BASH = Ench.R.enchant("shield_bash", (Supplier)ShieldBashEnchant::new);
            SPEARFISHING = Ench.R.enchant("spearfishing", (Supplier)SpearfishingEnchant::new);
            SPLITTING = Ench.R.enchant("splitting", (Supplier)SplittingEnchant::new);
            STABLE_FOOTING = Ench.R.enchant("stable_footing", (Supplier)StableFootingEnchant::new);
            TEMPTING = Ench.R.enchant("tempting", (Supplier)TemptingEnchant::new);
        }
    }
    
    public static class Tabs
    {
        public static final RegistryObject<CreativeModeTab> ENCH;
        
        private static void bootstrap() {
        }
        
        static {
            ENCH = Ench.R.tab("ench", () -> CreativeModeTab.builder().m_257941_((Component)Component.m_237115_("itemGroup.apotheosis.ench")).m_257737_(() -> ((BlockItem)Items.HELLSHELF.get()).m_7968_()).withTabsBefore(new ResourceLocation[] { Apotheosis.loc("adventure") }).m_257652_());
        }
    }
    
    public static class Tiles
    {
        public static final RegistryObject<BlockEntityType<FilteringShelfBlock.FilteringShelfTile>> FILTERING_SHELF;
        
        private static void bootstrap() {
        }
        
        static {
            FILTERING_SHELF = Ench.R.blockEntity("filtering_shelf", () -> new BlockEntityType(FilteringShelfBlock.FilteringShelfTile::new, (Set)ImmutableSet.of((Object)Blocks.FILTERING_SHELF.get()), (Type)null));
        }
    }
    
    public static class Colors
    {
        private static int[] _LIGHT_BLUE_FLASH;
        public static GradientColor LIGHT_BLUE_FLASH;
        
        static {
            Colors._LIGHT_BLUE_FLASH = new int[] { 46079, 46079, 46079, 46079, 46079, 46079, 46079, 46079, 46079, 46079, 46079, 46079, 46079, 46079, 46079, 46079, 46079, 46079, 46079, 46079, 46079, 46079, 46079, 46079, 46079, 46079, 46079, 46079, 46079, 46079, 46079, 46079, 46079, 46079, 46079, 767487, 1554687, 2276351, 2997759, 3784959, 4506623, 5228287, 6015487, 6737151 };
            Colors.LIGHT_BLUE_FLASH = new GradientColor(ApothMiscUtil.doubleUpGradient(Colors._LIGHT_BLUE_FLASH), "light_blue_flash");
        }
    }
}
